#fit the naive model with block averages of temperatures
rm(list=ls(all=TRUE))

dir0="C:/eco fal/"
dir1="C:/eco fal/GP_results/"
dir="C:/eco fal/results/"

library(pscl)
###################################################
#1. some useful functions
##(1.1)update eta
post_eta=function(theeta,k,W,eta_in,beta_in,X_in,tau_in,index)
{
 #theeta is the current value of eta[k], k the index of eta
 #if index=1, then X_in[k] is a number
 #if index>1, then X_in[[k]] is a vector, the grids in region k
 thesum=sum(W[k,])
 etabar=((W[k,]%*%eta_in)/thesum)[1,1]
 if(index==1)
 {
  theprob=(1+exp(-(beta_in[1]+beta_in[2]*X_in[k]+theeta)))^(-1)
 }
 if(index>1)
 {
  theprob=mean((1+exp(-(beta_in[1]+beta_in[2]*X_in[[k]]+theeta)))^(-1))
 }
 part1=0.5*tau_in*thesum*(theeta-etabar)^2
 part2=Y[k]*log(theprob)
 part3=(N[k]-Y[k])*log(1-theprob)
 res=part1*(-1)+part2+part3
 return(res)
}
update_eta=function(W,eta_in,beta_in,X_in,tau_in,omega_in,acc_eta_in,index)
{
  #update each eta[i]
  #omega is the variance for proposal dist
  for(k in 1:K)
  {
   #get proposal
   thecurr=eta_in[k]
   thestar=rnorm(1,thecurr,omega_in[k])
   logA=post_eta(thestar,k,W,eta_in,beta_in,X_in,tau_in,index)
   logB=post_eta(thecurr,k,W,eta_in,beta_in,X_in,tau_in,index)
   logrr=logA-logB
   logu=log(runif(1,0,1))
   if(logrr>logu)
   {
    eta_in[k]=thestar
    acc_eta_in[k]=acc_eta_in[k]+1
   }
  }
  #then centering eta
  eta_in=eta_in-mean(eta_in)
  return(list(eta_in,acc_eta_in))
}

##(1.2)update beta
post_beta=function(thebeta,i,eta_in,beta_in,X_in,index)
{
 #thebeta is the current value of beta[i], i the index of beta
 newbeta=beta_in
 newbeta[i]=thebeta
 if(index==1)
 {
  probmat=(1+exp(-(newbeta[1]+newbeta[2]*X_in+eta_in)))^(-1)
 }
 if(index>1)
 {
  probmat=rep(0,K)
  for(k in 1:K)
  {
   probmat[k]=mean((1+exp(-(newbeta[1]+newbeta[2]*X_in[[k]]+eta_in[k])))^(-1))
  }
 }
 a=Y*log(probmat)
 b=(N-Y)*log(1-probmat)
 part1=sum(a)
 part2=sum(b)
 res=part1+part2
 return(res)
}
update_beta=function(eta_in,beta_in,X_in,omega_in,acc_beta_in,index)
{
  #update each beta[i]
  #omega is the variance for proposal dist
  P=length(beta_in)
  for(i in 1:P)
  {
   #get proposal
   thecurr=beta_in[i]
   thestar=rnorm(1,thecurr,omega_in[i])
   logA=post_beta(thestar,i,eta_in,beta_in,X_in,index)
   logB=post_beta(thecurr,i,eta_in,beta_in,X_in,index)
   logrr=logA-logB
   logu=log(runif(1,0,1))
   if(logrr>logu)
   {
    beta_in[i]=thestar
    acc_beta_in[i]=acc_beta_in[i]+1
   }
  }
  return(list(beta_in,acc_beta_in))
}

post_beta2=function(thebeta,eta_in,X_in,index)
{
 #thebeta is the current value of beta[i], i the index of beta
 newbeta=thebeta
 if(index==1)
 {
  probmat=(1+exp(-(newbeta[1]+newbeta[2]*X_in+eta_in)))^(-1)
 }
 if(index>1)
 {
  probmat=rep(0,K)
  for(k in 1:K)
  {
   probmat[k]=mean((1+exp(-(newbeta[1]+newbeta[2]*X_in[[k]]+eta_in[k])))^(-1))
  }
 }
 a=Y*log(probmat)
 b=(N-Y)*log(1-probmat)
 part1=sum(a)
 part2=sum(b)
 res=part1+part2
 return(res)
}
update_beta2=function(eta_in,beta_in,X_in,omega_in,acc_beta_in,index)
{
  #update each beta[i]
  #omega is the variance for proposal dist
  P=length(beta_in)
  thestar=rep(0,P)
  for(i in 1:P)
  {
   #get proposal
   thestar[i]=rnorm(1,beta_in[i],omega_in[i])
  }
   logA=post_beta2(thestar,eta_in,X_in,index)
   logB=post_beta2(beta_in,eta_in,X_in,index)
   logrr=logA-logB
   logu=log(runif(1,0,1))
   if(logrr>logu)
   {
    beta_in=thestar
    acc_beta_in=acc_beta_in+1
   }
  return(list(beta_in,acc_beta_in))
}


##(1.3)update tau
update_tau=function(eta_in,a,b,W)
{
  #calculte(eta_i-eta_j)^2
  K=length(eta_in)
  etalist=rep(eta_in,K)
  p1=matrix(etalist,nrow=K,ncol=K,byrow=T)
  p2=matrix(etalist,nrow=K,ncol=K,byrow=F)
  pp=(p1-p2)^2
  ppp=pp[lower.tri(pp)]
  www=W[lower.tri(W)]
  newb=b+sum(ppp*www)/2
  newa=a+K/2
  newtau=rgamma(1,newa,newb)
  return(newtau)
}
#######################################end of usefull functions

####################################################
#2.fit GP using observed exposures and predict X values
##(2.1)read in the site of exposures
site_coor=read.csv(paste(dir0,"site_coor.csv",sep=""),header=T)
####read in the exposures in 1995
X_obs=read.csv(paste(dir0,"X_obs.csv",sep=""),header=T)
X_obs=sqrt(X_obs[,1])
X_M=X_obs
M=length(X_obs)
I_M=rep(1,M)
dist_M=matrix(0,nrow=M,ncol=M)
for(i in 1:M)
{
 for(j in 1:M)
 {
  lat1=site_coor[i,1]
  lon1=site_coor[i,2]
  lat2=site_coor[j,1]
  lon2=site_coor[j,2]
  dist_M[i,j]=sqrt((lat1-lat2)^2+(lon1-lon2)^2)
 }
}

##(2.2)use the estimated (mu,sigma2,phi) to predict X for each person
######(2.2.1)read in the locations of each individual
N=read.csv(paste(dir0,"N.csv",sep=""),header=T)
N=N[,1]
K=length(N)
Nsum=sum(N)

######(2.2.2)calculate two distance matrix, dist_L, dist_ML
X_area=read.csv(paste(dir1,"X_area.csv",sep=""),header=T)
X_area=X_area[,1]
X_area=(X_area-4.78)/0.2
##########################################end of GP

##########################################
#3.read in N, prepare matrices for saving, generate Y
##(3.1)some parameters
b0=-3.5
b1=0.1
P=2
W.mat=read.csv(paste(dir0,"W.mat.csv",sep=""),header=T)
W.mat=as.matrix(W.mat)
tau=0.1
eta_orig=read.csv(paste(dir0,"eta_orig.csv",sep=""),header=T)
eta_orig=eta_orig[,1]

Y=read.csv(paste(dir0,"Y.csv",sep=""),header=T)
Y=Y[,1]

##########################################end of generating data

  ##########################################
  #4.use the generated Y and X_M to fit model without correcting bias
  ##(4.1)predict X for each region
  ######since we already use X_M to fit GP, we just need to obtain the block average
  ######we do this before, in 2(5)
  ##(4.2)fit disease mapping model
  ######(4.2.1)some global parameters
  Atau=Btau=1
  n.T=10000
  n.burn0=5000
  n.burn=0
  n.tune=500
  times=10 #times for tune sigma
  step=2 #save posterior every stepth,so the total number is (n.T-n.burn)/step
  saveT1=(n.T-n.burn)/step
  etasave_w=matrix(0,nrow=saveT1,ncol=K)
  betasave_w=matrix(0,nrow=saveT1,ncol=P)
  tausave_w=rep(0,saveT1)
  probsave_w=matrix(0,nrow=saveT1,ncol=K)
  smrsave_w=matrix(0,nrow=saveT1,ncol=K)
  Esave_w=matrix(0,nrow=saveT1,ncol=K)
  ######(4.2.2)initalize
  tau=rgamma(1,Atau,Btau)
  eta=eta_orig
  omega0=0.7
  omega1=0.1
  omega11=0.005
  beta=c(-5,0.8)
  omega_in_eta=rep(0.3,K)
  omega_in_beta=c(0.05,0.01)
  beta2=rep(0,n.burn0)
  beta1=rep(0,n.burn0)

  ######(4.2.3)start with burn-in
  acc_eta=rep(0,K)  #count for acceptance
  acc_beta=rep(0,P)
  for(nn in 1:n.burn0)
  {
   ########update each eta using MH
   theres=update_eta(W.mat,eta,beta,X_area,tau,omega_in_eta,acc_eta,1)
   eta=theres[[1]]
   acc_eta=theres[[2]]
   ########update each beta using M-H
   theres=update_beta(eta,beta,X_area,omega_in_beta,acc_beta,1)
   beta=theres[[1]]
   beta2[nn]=beta[2]
   beta1[nn]=beta[1]
   acc_beta=theres[[2]]
   ########update tau
   tau=update_tau(eta,Atau,Btau,W.mat)
  }
   acc_eta=acc_eta/n.burn0
   acc_beta=acc_beta/n.burn0

  ######(4.2.4)model fitting
  acc_eta=rep(0,K)  #count for acceptance
  acc_beta=rep(0,P)
  for(nn in 1:n.T)
  {
   ########update each eta using MH
   theres=update_eta(W.mat,eta,beta,X_area,tau,omega_in_eta,acc_eta,1)
   eta=theres[[1]]
   acc_eta=theres[[2]]
   ########update each beta using M-H
   theres=update_beta(eta,beta,X_area,omega_in_beta,acc_beta,1)
   beta=theres[[1]]
   acc_beta=theres[[2]]
   ########update tau
   tau=update_tau(eta,Atau,Btau,W.mat)
   ########save posteriors after burnin
   if((nn>n.burn)&((nn-n.burn)%%step==0))
   {
    etasave_w[(nn-n.burn)/step,]=eta
    betasave_w[(nn-n.burn)/step,]=beta
    tausave_w[(nn-n.burn)/step]=tau
    aa=beta[1]+beta[2]*X_area+eta #linear prediction
    p=exp(aa)/(1+exp(aa))
    bb=sum(N*p)/sum(N)
    probsave_w[(nn-n.burn)/step,]=p
    smrsave_w[(nn-n.burn)/step,]=p/bb
    Esave_w[(nn-n.burn)/step,]=N*bb 
   }
  }
  acc_eta=acc_eta/n.T
  acc_beta=acc_beta/n.T
  print("end W")
  ####################################end for fit model using naive model

  ######################################
  #5.after model fitting, get saved values
  write.csv(etasave_w,paste(dir,"etasave_w.csv",sep=""),row.names=F)
  write.csv(betasave_w,paste(dir,"betasave_w.csv",sep=""),row.names=F)
  write.csv(tausave_w,paste(dir,"tausave_w.csv",sep=""),row.names=F)
  write.csv(probsave_w,paste(dir,"probsave_w.csv",sep=""),row.names=F)
  write.csv(smrsave_w,paste(dir,"smrsave_w.csv",sep=""),row.names=F)
  write.csv(Esave_w,paste(dir,"Esave_w.csv",sep=""),row.names=F)

  beta_w_each_mean=apply(betasave_w,2,mean)
  beta_w_each_med=apply(betasave_w,2,median)
  beta_w_each_sd=apply(betasave_w,2,sd)
  beta_w_each_lower=apply(betasave_w,2,function(x){return(quantile(x,0.025))})
  beta_w_each_upper=apply(betasave_w,2,function(x){return(quantile(x,0.975))})
  beta_save_w=cbind(beta_w_each_mean,beta_w_each_med,beta_w_each_sd,beta_w_each_lower,beta_w_each_upper)
  write.csv(beta_save_w,paste(dir,"beta_save_w.csv",sep=""),row.names=F) 
  prob_w_each_mean=apply(probsave_w,2,mean)
  prob_w_each_med=apply(probsave_w,2,median)
  prob_w_each_sd=apply(probsave_w,2,sd)
  prob_w_each_lower=apply(probsave_w,2,function(x){return(quantile(x,0.025))})
  prob_w_each_upper=apply(probsave_w,2,function(x){return(quantile(x,0.975))})
  prob_save_w=cbind(prob_w_each_mean,prob_w_each_med,prob_w_each_sd,prob_w_each_lower,prob_w_each_upper)
  write.csv(prob_save_w,paste(dir,"prob_save_w.csv",sep=""),row.names=F) 
  smr_w_each_mean=apply(smrsave_w,2,mean)
  smr_w_each_med=apply(smrsave_w,2,median)
  smr_w_each_sd=apply(smrsave_w,2,sd)
  smr_w_each_lower=apply(smrsave_w,2,function(x){return(quantile(x,0.025))})
  smr_w_each_upper=apply(smrsave_w,2,function(x){return(quantile(x,0.975))})
  smr_save_w=cbind(smr_w_each_mean,smr_w_each_med,smr_w_each_sd,smr_w_each_lower,smr_w_each_upper)
  write.csv(smr_save_w,paste(dir,"smr_save_w.csv",sep=""),row.names=F) 